@@ -1,9 +1,9 @@
 import osc.core
 import osc.oscerr
 import os
-from common import OscTestCase
+from .common import OscTestCase
 
-FIXTURES_DIR = os.path.join(os.getcwd(), 'project_package_status_fixtures')
+FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'project_package_status_fixtures')
 
 def suite():
     import unittest
