@@ -2,9 +2,15 @@ import osc.core
 import osc.oscerr
 import os
 import sys
-from common import GET, PUT, POST, DELETE, OscTestCase
-from xml.etree import cElementTree as ET
-FIXTURES_DIR = os.path.join(os.getcwd(), 'repairwc_fixtures')
+from .common import GET, PUT, POST, DELETE, OscTestCase
+try:
+    # Works up to Python 3.8, needed for Python < 3.3 (inc 2.7)
+    from xml.etree import cElementTree as ET
+except ImportError:
+    # will import a fast implementation from 3.3 onwards, needed
+    # for 3.9+
+    from xml.etree import ElementTree as ET
+FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'repairwc_fixtures')
 
 def suite():
     import unittest
